Package: nhppp
Title: Simulating Nonhomogeneous Poisson Point Processes
Version: 1.0.5
Authors@R: 
    c(person(given = "Thomas", 
             family = "Trikalinos", 
             email = "thomas_trikalinos@brown.edu", 
             role = c("aut", "cre", "cph"),
             comment = c(ORCID = "0000-0002-3990-1848")), 
      person(given = "Yuliia", 
             family = "Sereda", 
             email = "sereda_yuliia@brown.edu", 
             role = c("aut"),
             comment = c(ORCID = "0000-0002-4017-4561"))
    )
Description: Simulates events from one dimensional nonhomogeneous Poisson point processes (NHPPPs) as per Trikalinos and Sereda (2024, <doi:10.48550/arXiv.2402.00358> and 2024, <doi:10.1371/journal.pone.0311311>). Functions are based on three algorithms that provably sample from a target NHPPP: the time-transformation of a homogeneous Poisson process (of intensity one) via the inverse of the integrated intensity function (Cinlar E, "Theory of stochastic processes" (1975, ISBN:0486497996)); the generation of a Poisson number of order statistics from a fixed density function; and the thinning of a majorizing NHPPP via an acceptance-rejection scheme (Lewis PAW, Shedler, GS (1979) <doi:10.1002/nav.3800260304>).
License: GPL (>= 3)
Imports: lifecycle, rstream, Rcpp (>= 1.0.12)
LinkingTo: Rcpp
Encoding: UTF-8
Language: en
RoxygenNote: 7.3.2
Suggests: data.table, ggplot2, knitr, rlecuyer, rmarkdown, testthat,
        tictoc, truncnorm, withr,
Config/Needs/website: rmarkdown
URL: https://bladder-ca.github.io/nhppp/,
        https://github.com/bladder-ca/nhppp
BugReports: https://github.com/bladder-ca/nhppp/issues
VignetteBuilder: knitr
Depends: R (>= 2.10)
LazyData: true
NeedsCompilation: yes
Packaged: 2026-04-21 23:00:51 UTC; tom
Author: Thomas Trikalinos [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-3990-1848>),
  Yuliia Sereda [aut] (ORCID: <https://orcid.org/0000-0002-4017-4561>)
Maintainer: Thomas Trikalinos <thomas_trikalinos@brown.edu>
Repository: CRAN
Date/Publication: 2026-04-21 23:22:09 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-22 00:08:08 UTC; unix
Archs: nhppp.so.dSYM
