# Generated by roxygen2: do not edit by hand

export(draw)
export(draw_cumulative_intensity)
export(draw_intensity)
export(draw_sc_linear)
export(draw_sc_loglinear)
export(draw_sc_step)
export(draw_sc_step_regular)
export(get_step_majorizer)
export(inverse_with_uniroot)
export(inverse_with_uniroot_sorted)
export(ppp)
export(ppp_exactly_n)
export(ppp_n)
export(ppp_next_n)
export(ppp_orderstat)
export(ppp_sequential)
export(rng_stream_rexp)
export(rng_stream_rpois)
export(rng_stream_runif)
export(rng_stream_rztpois)
export(rztpois)
export(simpson_num_integr)
export(vdraw)
export(vdraw_cumulative_intensity)
export(vdraw_intensity)
export(vdraw_sc_step_regular)
export(vdraw_sc_step_regular_cpp)
export(ztdraw_cumulative_intensity)
export(ztdraw_sc_linear)
export(ztdraw_sc_loglinear)
export(ztppp)
importClassesFrom(rstream,rstream.mrg32k3a)
importFrom(Rcpp,sourceCpp)
importFrom(lifecycle,deprecated)
useDynLib(nhppp, .registration = TRUE)
