# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

find_lower_bound_index <- function(L, start, tau) {
    .Call(`_nhppp_find_lower_bound_index`, L, start, tau)
}

find_upper_bound_index <- function(L, start, tau) {
    .Call(`_nhppp_find_upper_bound_index`, L, start, tau)
}

matrix_cumsum_columns <- function(X) {
    .Call(`_nhppp_matrix_cumsum_columns`, X)
}

matrix_cumsum_columns_inplace <- function(X) {
    invisible(.Call(`_nhppp_matrix_cumsum_columns_inplace`, X))
}

matrix_diff_columns <- function(X) {
    .Call(`_nhppp_matrix_diff_columns`, X)
}

matrix_diff_columns_inplace <- function(X) {
    invisible(.Call(`_nhppp_matrix_diff_columns_inplace`, X))
}

rztpois <- function(lambda) {
    .Call(`_nhppp_rztpois`, lambda)
}

rztpois_vec <- function(lambda) {
    .Call(`_nhppp_rztpois_vec`, lambda)
}

step_regular_inverse <- function(max_events, Lambda, Tau, range_t, atmost1) {
    .Call(`_nhppp_step_regular_inverse`, max_events, Lambda, Tau, range_t, atmost1)
}

vdraw_intensity_step_regular <- function(lambda, rate_maj, is_cumulative, range_t, subinterval, use_subinteval, tol, atmost1, atmostB) {
    .Call(`_nhppp_vdraw_intensity_step_regular`, lambda, rate_maj, is_cumulative, range_t, subinterval, use_subinteval, tol, atmost1, atmostB)
}

vdraw_sc_step_regular <- function(rate, is_cumulative, range_t, tol, atmost1) {
    .Call(`_nhppp_vdraw_sc_step_regular`, rate, is_cumulative, range_t, tol, atmost1)
}

vdraw_sc_step_regular2 <- function(rate, is_cumulative, range_t, subinterval, tol, atmost1, atmostB) {
    .Call(`_nhppp_vdraw_sc_step_regular2`, rate, is_cumulative, range_t, subinterval, tol, atmost1, atmostB)
}

vztdraw_sc_step_regular <- function(rate, is_cumulative, range_t, atmost1) {
    .Call(`_nhppp_vztdraw_sc_step_regular`, rate, is_cumulative, range_t, atmost1)
}

vztdraw_sc_step_regular2 <- function(rate, is_cumulative, range_t, subinterval, atmost1) {
    .Call(`_nhppp_vztdraw_sc_step_regular2`, rate, is_cumulative, range_t, subinterval, atmost1)
}

