% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matrix_diff_columns.R
\name{matrix_diff_columns}
\alias{matrix_diff_columns}
\title{Usage: matrix_diff_columns( X )}
\usage{
matrix_diff_columns(X)
}
\description{
Usage: matrix_diff_columns( X )
}
\keyword{internal}
