% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-rngstream.R
\name{rztpois}
\alias{rztpois}
\title{Zero-truncated Poisson random samples (basic R)}
\usage{
rztpois(n, lambda)
}
\arguments{
\item{n}{Integer, number of samples}

\item{lambda}{Positive number, the mean of the original
(untruncated) Poisson distribution}
}
\value{
a vector of non zero counts of size \code{n}
}
\description{
Sample zero-truncated Poisson random samples (basic R)
}
\examples{
rztpois(10, 1)
rztpois(10, 1:10)
}
\keyword{internal}
